from django import forms
from django.contrib.auth import get_user_model
from authX.authX_models import AuthXAppUserModel
from django.contrib.auth.forms import UserCreationForm

class AuthXLoginUserForm(forms.Form):
    username = forms.CharField(
        widget=forms.TextInput(
            attrs={
                'class': 'form-control',
                'placeholder': 'Username'
            }
        ),
        help_text='Enter your username',
        required=True
    )
    password = forms.CharField(
        widget=forms.PasswordInput(
            attrs={
                'class': 'form-control',
                'placeholder': 'Password'
            }
        ),
        help_text='Enter your password',
        required=True
    )

class AuthXRegisterUserForm(UserCreationForm):

    def __init__(self, *args, **kwargs):
        super(AuthXRegisterUserForm, self).__init__(*args, **kwargs)
        self.fields['username'].widget.attrs.update({'class': 'form-control'})
        self.fields['email'].widget.attrs.update({'class': 'form-control'})

    class Meta:
        model = AuthXAppUserModel
        fields = ['username','email',]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def clean_email(self):
        email = self.cleaned_data.get('email')
        return email
    
    def clean_username(self):
        username = self.cleaned_data.get('username')
        return username


AUTHX_CONSTANT_CONFIGS = {
    'AUTHX_USER_MODEL' : 'authX.AuthXAppUserModel',
    'AUTHX_LOGIN_URL' : 'login/',
    'AUTHX_REGISTER_URL' : 'register/',
    'AUTHX_LOGOUT_URL' : 'logout/',
    'AUTHX_DASHBOARD_URL' : '/dashboard/',
}



from django.contrib import admin
from authX.authX_models import AuthXAppUserModel
from django.contrib.auth.admin import UserAdmin as OriginalAdmin
from django.utils.translation import gettext_lazy as _


class AuthXAppUserAdmin(OriginalAdmin):

    list_display = ("id", "username", "email", "date_joined", "last_login")

    list_display_links = ("username",)
    list_per_page = 25
    search_fields = ("username", "email")
    list_filter = ("date_joined", "last_login")

    fieldsets = (
        (
            _("Login Information"),
            {
                "fields": (
                    "username",
                    "password",
                )
            },
        ),
        (
            _("Personal Information"),
            {
                "fields": (
                    "first_name",
                    "last_name",
                    "email",
                    "phone_number_prefix",
                    "phone_number",
                    "user_profile_picture",
                    "birth_date",
                )
            },
        ),
        (
            _("Permissions"),
            {
                "fields": (
                    "is_active",
                    "is_staff",
                    "is_superuser",
                    "groups",
                    "user_permissions",
                ),
            },
        ),
        (
            _("Security Features"),
            {
                "fields": (
                    "security_question",
                    "security_question_answer",
                    "failed_login_attempts",
                ),
            },
        ),
        (
            _("Important Dates"),
            {
                "fields": (
                    "last_login",
                    "date_joined",
                )
            },
        ),
    )


admin.site.register(AuthXAppUserModel, AuthXAppUserAdmin)



from django.shortcuts import redirect
from django.urls import reverse_lazy
from authX.authX_settings import AUTHX_SETTINGS

class LoginRequiredMixin:
    redirect_url = reverse_lazy(AUTHX_SETTINGS['AUTHX_LOGIN_URL'])

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_authenticated:
            return redirect(self.redirect_url)
        return super().dispatch(request, *args, **kwargs)

class RedirectIfAuthenticatedMixin:
    redirect_url = reverse_lazy(AUTHX_SETTINGS['AUTHX_DASHBOARD_URL'])

    def dispatch(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            return redirect(self.redirect_url)
        return super().dispatch(request, *args, **kwargs)


from authX.authX_settings import AUTHX_SETTINGS
from django.shortcuts import redirect

def already_logged_in(view_func):
    def wrapper_func(request, *args, **kwargs):
        if not request.user.is_authenticated:
            return view_func(request, *args, **kwargs)
        else:
            return redirect(AUTHX_SETTINGS['AUTHX_LOGIN_URL'])
    
    return wrapper_func


from django.shortcuts import render
from django.views.generic import FormView
from authX.authX_forms import AuthXLoginUserForm, AuthXRegisterUserForm
from django.contrib.auth import authenticate, login
from django.contrib.auth.views import LogoutView
from authX.authX_settings import AUTHX_SETTINGS
from authX.authX_mixins import RedirectIfAuthenticatedMixin
from django.urls import reverse_lazy

class AuthXRegisterView(RedirectIfAuthenticatedMixin, FormView):
    template_name = 'authX_templates/register.html'
    form_class = AuthXRegisterUserForm
    success_url = AUTHX_SETTINGS['AUTHX_LOGIN_URL']
    
    def form_valid(self, form):
        username = form.cleaned_data['username']
        password = form.cleaned_data['password']
        user = authenticate(username=username, password=password)
        if user is not None:
            login(self.request, user)
            return super().form_valid(form)
        else:
            return self.form_invalid(form)
    
class AuthXLogoutView(LogoutView):
    next_page = reverse_lazy('authX_login')


class AuthXLoginView(RedirectIfAuthenticatedMixin, FormView):
    template_name = 'authX_templates/login.html'
    form_class = AuthXLoginUserForm
    success_url = AUTHX_SETTINGS['AUTHX_DASHBOARD_URL']

    def form_valid(self, form):
        username = form.cleaned_data['username']
        password = form.cleaned_data['password']
        user = authenticate(username=username, password=password)
        if user is not None:
            login(self.request, user)
            return super().form_valid(form)
        else:
            return self.form_invalid(form)


from django.conf import settings
from authX.authX_constants import AUTHX_CONSTANT_CONFIGS

AUTHX_SETTINGS =  {
    'AUTHX_USER_MODEL': getattr(settings,'AUTHX_USER_MODEL', AUTHX_CONSTANT_CONFIGS['AUTHX_USER_MODEL']),
    'AUTHX_LOGIN_URL' : getattr(settings,'AUTHX_LOGIN_URL', AUTHX_CONSTANT_CONFIGS['AUTHX_LOGIN_URL']),
    'AUTHX_LOGOUT_URL': getattr(settings,'AUTHX_LOGOUT_URL', AUTHX_CONSTANT_CONFIGS['AUTHX_LOGOUT_URL']),
    'AUTHX_REGISTER_URL': getattr(settings,'AUTHX_REGISTER_URL', AUTHX_CONSTANT_CONFIGS['AUTHX_REGISTER_URL']),
    'AUTHX_DASHBOARD_URL': getattr(settings,'AUTHX_DASHBOARD_URL', AUTHX_CONSTANT_CONFIGS['AUTHX_DASHBOARD_URL']),
}


from django.urls import path
from authX.authX_views import AuthXRegisterView, AuthXLoginView,AuthXLogoutView
from authX.authX_settings import AUTHX_SETTINGS

urlpatterns = [
    path(AUTHX_SETTINGS['AUTHX_LOGIN_URL'],AuthXLoginView.as_view(),name="authX_login"),
    path(AUTHX_SETTINGS['AUTHX_REGISTER_URL'],AuthXRegisterView.as_view(),name="authX_register"),
    path(AUTHX_SETTINGS['AUTHX_LOGOUT_URL'],AuthXLogoutView.as_view(),name="authX_logout"),
]


from django.db import models
from django.contrib.auth.models import AbstractUser, Group, Permission
from datetime import datetime

class AuthXAppUserModel(AbstractUser):
    first_name = models.CharField(
        max_length=50,
        unique=False,
        help_text="First name",
        null=True,
        blank=True,
    )
    last_name = models.CharField(
        max_length=50,
        unique=False,
        help_text="Last name",
        null=True,
        blank=True,
    )
    username = models.CharField(
        max_length=50,
        unique=True,
        help_text="Username",
        error_messages= {
            "unique": "A user with that username already exists.",
            "required": "This field cannot be blank."
        }
    )
    email = models.EmailField(
        max_length=255,
        unique=True,
        help_text="Email address",
        error_messages= {
            "unique": "A user with that email already exists.",
            "required": "This field cannot be blank."
        }
    )
    phone_number_prefix = models.CharField(
        max_length=10,
        unique=False,
        help_text="Phone number prefix (e.g., +1, +63, etc.)",
        null=True,
        blank=True,
    )
    phone_number = models.CharField(
        max_length=15,
        unique=True,
        help_text="Phone number (e.g., 09050673466)",
        null=True,
        blank=True,
    )
    failed_login_attempts = models.PositiveIntegerField(
        default=0, help_text="Failed login attempts"
    )
    password_last_updated = models.DateTimeField(
        auto_now=True, help_text="Last password update timestamp"
    )
    security_question = models.CharField(
        max_length=255,
        unique=False,
        help_text="Security question",
        null=True,
        blank=True,
    )
    security_question_answer = models.CharField(
        max_length=255,
        unique=False,
        help_text="Security question answer",
        null=True,
        blank=True,
    )
    user_profile_picture = models.ImageField(
        upload_to="user_profile_pictures/",
        help_text="User profile picture",
        null=True,
        blank=True,
    )
    birth_date = models.DateField(
        null=True,
        blank=True,
        help_text="Date of birth",
    )

    class Meta:
        db_table = "auth_user"
        verbose_name = "User"
        verbose_name_plural = "Users"

    def __str__(self) -> str:
        return self.username
    
    def getUserName(self) -> str:
        return self.username
    
    def getFullname(self) -> str:
        return f'{self.first_name} {self.last_name}'
    
    def getPhoneNumber(self) -> str:
        return f'{self.phone_number_prefix} {self.phone_number}'
    
    def isBirthday(self) -> bool:
        today = datetime.date.today()
        return today.month == self.birth_date.month and today.day == self.birth_date.day
    
